package com.yuandian.dataflow.statemachine.closure;

import com.alipay.sofa.jraft.Closure;
import com.alipay.sofa.jraft.Status;
import com.alipay.sofa.jraft.entity.PeerId;
import com.yuandian.dataflow.statemachine.StateFactory;
import com.yuandian.dataflow.statemachine.operate.Operate;
import com.yuandian.dataflow.statemachine.rpc.RaftResponse;
import com.yuandian.dataflow.statemachine.state.State;

import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Slf4j
@Getter
@Setter
@ToString
public  abstract  class  GenericClosure<T>  implements Closure {
 
 
	// 状态机的统一响应
	private RaftResponse<T> response;
	// 代表任务状态 
	private T value;

    public GenericClosure() {
  
    }

	/**
     * 错误的时候返回错误信息.  自动装配response
	 * @param errorMsg
	 * @param redirect
	 */
	public void failure(final String errorMsg, final PeerId redirect) {
        final RaftResponse<T> response = new RaftResponse<T>();
        response.setSuccess(false);
        response.setMsg(errorMsg);
        response.setRedirect(redirect);
        setResponse(response);
    }

    /**
     * 成功时调用该方法. 自动装配response
     * @param value
     */
    public void success(final T value) {
        final RaftResponse<T> response = new RaftResponse<T>();
        response.setValue(value);
        response.setSuccess(true);
        setResponse(response);
    }
}
